/* ARCADEmini simulator (unix, console)
 * version 1.0 date 2004-11-21
 * part of the ARCADEmini project - http[s]://arcademini.schuermans.info/
 * Copyright (C) 2003-2004 sphaera <sphaera@geekness.de>
 *                     and 1stein <1stein@schuermans.info>
 * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
 */

#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <termios.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "am_dev.h"

struct termios stdin_settings_orig, stdin_settings_new;

//initialize input
//returns 0 on success, -1 on error
int input_init( )
{
  //set stdin to unbuffered mode
  tcgetattr( 0, &stdin_settings_orig );
  memcpy( &stdin_settings_new, &stdin_settings_orig, sizeof( struct termios ) );
  stdin_settings_new.c_lflag &= ~ICANON & ~ECHO;
  stdin_settings_new.c_cc[VMIN] = 1;
  stdin_settings_new.c_cc[VTIME] = 0;
  tcsetattr( 0, TCSANOW, &stdin_settings_new );

  return 0;
}

//polling procedure for input
//returns true to signal end of simulation
int input_poll( st_am_state * p_state )
{
  int input_cnt;
  unsigned char input;

  //process input
  for( ioctl( 0, FIONREAD, &input_cnt ); input_cnt > 0; input_cnt-- )
  {
    //get next byte of input
    if( read( 0, &input, 1 ) == 1 )
    {
      switch( input )
      {
        case '1': p_state->keys_pressed |= 0x01; p_state->keys_released |= 0x01; break;
        case '2': p_state->keys_pressed |= 0x02; p_state->keys_released |= 0x02; break;
        case '3': p_state->keys_pressed |= 0x04; p_state->keys_released |= 0x04; break;
        case '4': p_state->keys_pressed |= 0x08; p_state->keys_released |= 0x08; break;
        case '5': p_state->keys_pressed |= 0x10; p_state->keys_released |= 0x10; break;
        case '6': p_state->keys_pressed |= 0x20; p_state->keys_released |= 0x20; break;
        case '7': p_state->keys_pressed |= 0x40; p_state->keys_released |= 0x40; break;
        case '8': p_state->keys_pressed |= 0x80; p_state->keys_released |= 0x80; break;
        case 0x1B: return 1;
      }
    }
  }
  return 0;
}

//cleanup input
void input_exit( )
{
  //restore stdin settings
  tcsetattr( 0, TCSANOW, &stdin_settings_orig );
}

